@@ -6,12 +6,14 @@ use serde::{Deserialize, Serialize};
 /// The transfer function is I^2,I from input phase to output phase and P,I from input phase to
 /// output frequency.
 ///
+/// The transfer functions (for phase and frequency) contain an additional zero at Nyquist.
+///
 /// The PLL locks to any frequency (i.e. it locks to the alias in the first Nyquist zone) and is
 /// stable for any gain (1 <= shift <= 30). It has a single parameter that determines the loop
 /// bandwidth in octave steps. The gain can be changed freely between updates.
 ///
 /// The frequency and phase settling time constants for a frequency/phase jump are `1 << shift`
-/// update cycles. The loop bandwidth is about `1/(2*pi*(1 << shift))` in units of the sample rate.
+/// update cycles. The loop bandwidth is `1/(2*pi*(1 << shift))` in units of the sample rate.
 /// While the phase is being settled within one turn, there is a typically very small frequency
 /// overshoot.
 ///
@@ -24,7 +26,7 @@ use serde::{Deserialize, Serialize};
 /// bias is applied. Rounding is "half up". The phase truncation error can be removed very
 /// efficiently by dithering.
 ///
-/// This PLL does not unwrap phase slips during lock acquisition. This can and should be
+/// This PLL does not unwrap phase slips accumulated during lock acquisition. This can and should be
 /// implemented elsewhere by unwrapping and scaling the input phase and un-scaling
 /// and wrapping output phase and frequency. This affects dynamic range, gain, and noise accordingly.
 ///
@@ -52,31 +54,32 @@ impl PLL {
     ///   per update. A good value is typically `shift_frequency - 1`.
     ///
     /// Returns:
-    /// A tuple of instantaneous phase and frequency (the current phase increment).
-    pub fn update(&mut self, x: Option<i32>, shift_frequency: u8, shift_phase: u8) -> (i32, i32) {
+    /// A tuple of instantaneous phase and frequency.
+    pub fn update(&mut self, x: Option<i32>, shift_frequency: u32, shift_phase: u32) -> (i32, i32) {
         debug_assert!((1..=30).contains(&shift_frequency));
         debug_assert!((1..=30).contains(&shift_phase));
-        let f = if let Some(x) = x {
-            let e = x.wrapping_sub(self.f);
-            self.f = self.f.wrapping_add(
-                (1i32 << (shift_frequency - 1))
-                    .wrapping_add(e)
-                    .wrapping_sub(self.x)
-                    >> shift_frequency,
-            );
+        if let Some(x) = x {
+            let df = (1i32 << (shift_frequency - 1))
+                .wrapping_add(x)
+                .wrapping_sub(self.x)
+                .wrapping_sub(self.f)
+                >> shift_frequency;
             self.x = x;
-            self.f.wrapping_add(
-                (1i32 << (shift_phase - 1))
-                    .wrapping_add(e)
-                    .wrapping_sub(self.y)
-                    >> shift_phase,
-            )
+            self.f = self.f.wrapping_add(df);
+            let f = self.f.wrapping_sub(df >> 1);
+            self.y = self.y.wrapping_add(f);
+            let dy = (1i32 << (shift_phase - 1))
+                .wrapping_add(x)
+                .wrapping_sub(self.y)
+                >> shift_phase;
+            self.y = self.y.wrapping_add(dy);
+            let y = self.y.wrapping_sub(dy >> 1);
+            (y, f)
         } else {
             self.x = self.x.wrapping_add(self.f);
-            self.f
-        };
-        self.y = self.y.wrapping_add(f);
-        (self.y, f)
+            self.y = self.y.wrapping_add(self.f);
+            (self.y, self.f)
+        }
     }
 }
 
@@ -87,8 +90,8 @@ mod tests {
     fn mini() {
         let mut p = PLL::default();
         let (y, f) = p.update(Some(0x10000), 8, 4);
-        assert_eq!(y, 0x1100);
-        assert_eq!(f, y);
+        assert_eq!(y, 0x87c);
+        assert_eq!(f, 0x80);
     }
 
     #[test]
@@ -102,7 +105,8 @@ mod tests {
             x = x.wrapping_add(f0);
             let (y, f) = p.update(Some(x), shift.0, shift.1);
             if i > n / 4 {
-                assert_eq!(f.wrapping_sub(f0).abs() <= 1, true);
+                // The remaining error would be removed by dithering.
+                assert_eq!(f.wrapping_sub(f0).abs() <= 1 << 10, true);
             }
             if i > n / 2 {
                 // The remaining error would be removed by dithering.
